@@ -88,8 +88,10 @@
           'Confirm Delete Load Balancers', count),
 
         message: ngettext(
-          'You have selected "%s". Deleted load balancer is not recoverable.',
-          'You have selected "%s". Deleted load balancers are not recoverable.', count),
+          'You have selected "%s". Deleted load balancer is not recoverable ' +
+          'and this deletion will delete all of the sub-resources.',
+          'You have selected "%s". Deleted load balancers are not recoverable ' +
+          'and this deletion will delete all of the sub-resources.', count),
 
         submit: ngettext(
           'Delete Load Balancer',
@@ -124,9 +126,9 @@
     }
 
     function allowed() {
-      // This rule is made up and should therefore always pass. I assume at some point there
-      // will be a valid rule similar to this that we will want to use.
-      return policy.ifAllowed({ rules: [['neutron', 'delete_loadbalancer']] });
+      return policy.ifAllowed({
+        rules: [['load-balancer', 'os_load-balancer_api:loadbalancer:delete']]
+      });
     }
 
     function canBeDeleted(item) {
