@@ -42,7 +42,9 @@
         weight: 1,
         monitor_address: '1.1.1.1',
         monitor_port: 80,
-        admin_state_up: true
+        admin_state_up: true,
+        backup: false,
+        name: 'member name'
       });
       $provide.value('horizon.app.core.openstack-service-api.lbaasv2', {
         editMember: function() {
@@ -69,7 +71,7 @@
       expect(ctrl.weight).toBe(1);
       expect(ctrl.ipPattern).toBeDefined();
       expect(ctrl.helpUrl).toBeDefined();
-      expect(ctrl.weightError).toBe('The weight must be a number between 1 and 256.');
+      expect(ctrl.weightError).toBe('The weight must be a number between 0 and 256.');
       expect(ctrl.monitorAddressError).toBe('The monitor address must be a valid IP address.');
       expect(ctrl.monitorPortError).toBe('The monitor port must be a number between 1 and 65535.');
     });
@@ -84,7 +86,9 @@
         weight: 1,
         monitor_address: '1.1.1.1',
         monitor_port: 80,
-        admin_state_up: true
+        admin_state_up: true,
+        backup: false,
+        name: 'member name'
       });
       expect($uibModalInstance.close).toHaveBeenCalled();
     });
